﻿/************************************************************
 *	File:		RailroadResources.cs
 *
 * 	Author:		Samuel Joski
 *	
 *	Contents:	Class Train
 *				Class Turnout
 * 	
 * 	Purpose:	These classes are responsible for holding all
 * 				information about railroad resources.
 * 				
 * 				These classes differ from the thsoe found in
 * 				CCRServer's RailroadResources.cs, as these are
 * 				used mainly as structs to hold and modify
 * 				data, as well as this class does not do any
 * 				computation of the data to determine the next
 * 				state of the CCR.			
 * *********************************************************/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MasterCAB
{
	class Train
	{
		private int id;
		private int address;
		private int location;
		private int speed;
		private int direction;
		private int orientation;
		private int status;
		private bool headlight;
		private bool horn;
		private bool occupied;
		
		//Constructors
		public Train(int id)
		{
			this.id = id;
			address = -1;
			location = -1;
			speed = 0;
			direction = GLOBALS.LEFT;
			orientation = GLOBALS.LEFT;
			status = GLOBALS.HALTED;
			headlight = false;
			horn = false;
			occupied = false;
		}
		public Train(int id, int address, int location, int speed, int direction, int orientation, int status, bool headlight, bool horn, bool occupied)
		{
			this.id = id;
			this.address = address;
			this.location = location;
			this.speed = speed;
			this.direction = direction;
			this.orientation = orientation;
			this.status = status;
			this.headlight = headlight;
			this.horn = horn;
			this.occupied = occupied;
		}

		//Accessors
		public int ID() { return id; }
		public int Address() { return address; }
		public int Location() { return location; }
		public int Speed() { return speed; }
		public int Direction() { return direction; }
		public int Orientation() { return orientation; }
		public int Status() { return status; }
		public bool Headlight() { return headlight; }
		public bool Horn() { return horn; }
		public bool isOccupied() { return occupied; }
		public string GetInfoLine()
		{
			return (
				id.ToString() + ',' +
				address.ToString() + ',' +
				location.ToString() + ',' +
				speed.ToString() + ',' +
				direction.ToString() + ',' +
				orientation.ToString() + ',' +
				status.ToString() + ',' +
				Convert.ToInt32(headlight).ToString() + ',' +
				Convert.ToInt32(horn).ToString() + ',' +
				Convert.ToInt32(occupied).ToString()
			);
		}

		//Mutators
		public void setAddress(int address) { this.address = address; }
		public void setLocation(int location) { this.location = location; }
		public void setSpeed(int speed) { this.speed = speed; }
		public void setDirection(int direction) { this.direction = direction; }
		public void setOrientation(int orientation) { this.orientation = orientation; }
		public void setStatus(int status) { this.status = status; }
		public void setHeadlight(bool headlight) { this.headlight = headlight; }
		public void setHorn(bool horn) { this.horn = horn; }
		public void setOccupied(bool occupied) { this.occupied = occupied; }
		public bool Occupy()
		{
			if(!occupied)
			{
				occupied = true;
				return true;
			}
			return false;
		}
		public bool Release()
		{
			if(occupied)
			{
				occupied = false;
				return true;
			}
			return false;
		}

	}
	class Turnout
	{
		private int id;
		private int address;
		private int direction;
		private bool occupied;

		//Constructors
		public Turnout(int id)
		{
			this.id = id;
			address = -1;
			direction = GLOBALS.TOP;
			occupied = false;
		}
		public Turnout(int id, int address, int direction, bool occupied)
		{
			this.id = id;
			this.address = address;
			this.direction = direction;
			this.occupied = occupied;
		}

		//Accessors
		public int ID() { return id; }
		public int Address() { return address; }
		public int Direction() { return direction; }
		public bool Occupied() { return occupied; }
		public string GetInfoLine()
		{
			return (
				id.ToString() + ',' +
				address.ToString() + ',' +
				direction.ToString() + ',' +
				Convert.ToInt32(occupied).ToString()
			);
		}

		//Mutators
		public void setAddress(int address) { this.address = address; }
		public void setDirection(int direction) { this.direction = direction; }
		public void setOccupied(bool occupied) { this.occupied = occupied; }

	}
}
